/* interface.c
 * AbsVolume was programmed by Paul Sherman, derived in near 
 * entirety from the program "volumecontrol" by Rob Eberts
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License v2.
 * source code is available at <http://www.pcbypaul.com>
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"

#include <linux/soundcard.h>
#include <fcntl.h>

static char * volume_xpm[] = {
"16 16 7 1",
" 	c None",
".	c #000000",
"+	c #808000",
"@	c #808080",
"#	c #FFFF00",
"$	c #C0C0C0",
"%	c #FFFFFF",
"       ..       ",
"      +@.@      ",
"     +#.$$      ",
"    +#@@$%@   @ ",
"   +#%.$%%@  @  ",
" ++#%#.$%%@ @   ",
"+%$%#%..%%@     ",
"+#$#%#.$.%@     ",
"+#$%#%.@.%@ @@@@",
".+@#%#..%%@     ",
" ..+#%.$%%@     ",
"   .+#.$%%@ @   ",
"    .+@@%%@  @  ",
"     .+.$$@   @ ",
"      .+.@      ",
"       ..       "
};


extern int mixer_fd;
typedef struct stereovolume
{
  unsigned char left;
  unsigned char right;
} StereoVolume;
StereoVolume tmpvol;

struct originalset orig;


GtkWidget *window1;
GtkWidget *vbox1;
GtkWidget *hbox1;
GtkWidget *spinbutton1;
GtkWidget *hbox3;
GtkWidget *vscale1;
GtkAccelGroup *accel_group;
GtkAdjustment *vol_adjustment;
GtkWidget *hbox4;
GtkWidget *closeBtn;
GdkPixbuf *icon;
GError *error=NULL;
GtkWidget *frame;

GtkWidget*
create_window1 (void)
{

  accel_group = gtk_accel_group_new ();

  window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_size_request (window1, 60, 240);
  gtk_container_set_border_width (GTK_CONTAINER (window1), 2);

  gtk_window_set_resizable (GTK_WINDOW (window1), FALSE);
  gtk_window_set_position (GTK_WINDOW (window1),GTK_WIN_POS_MOUSE);
  gtk_window_set_decorated (GTK_WINDOW (window1), FALSE);
  
  
  icon = gdk_pixbuf_new_from_xpm_data(volume_xpm);
  if(icon) {
	gtk_window_set_icon(GTK_WINDOW (window1), icon);
  } else {
        g_error_free(error);
  }

  frame = gtk_frame_new ("Volume");

  
  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (frame), vbox1);
  gtk_container_add (GTK_CONTAINER (window1), frame);
  gtk_widget_show (frame);
  vol_adjustment=GTK_ADJUSTMENT (gtk_adjustment_new (1, 0, 100, 1, 10, 0));

  /* get original adjustments */
  get_current_levels();

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, TRUE, 8);


  spinbutton1 = gtk_spin_button_new (GTK_ADJUSTMENT (vol_adjustment),1,0); /* ajustment, climb rate, digits */
  gtk_widget_show (spinbutton1);
  gtk_box_pack_start (GTK_BOX (hbox1), spinbutton1, TRUE, FALSE, 0);
  gtk_widget_set_size_request (spinbutton1, -1, 22);
  g_signal_connect ((gpointer) spinbutton1, "value_changed",
                    G_CALLBACK (on_spinbutton1_button_release_event),
                    NULL);
  g_object_set_data_full(G_OBJECT (window1), "spinbutton1",gtk_widget_ref (spinbutton1),(GDestroyNotify) gtk_widget_unref);

 


 
  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox3, TRUE, TRUE, 5);

  vscale1 = gtk_vscale_new (GTK_ADJUSTMENT (vol_adjustment));
  gtk_widget_show (vscale1);
  gtk_range_set_inverted (GTK_RANGE (vscale1), TRUE);
  gtk_box_pack_start (GTK_BOX (hbox3), vscale1, TRUE, TRUE, 0);
  gtk_scale_set_draw_value (GTK_SCALE (vscale1), FALSE);
  g_signal_connect ((gpointer) vscale1, "button_release_event",
                    G_CALLBACK (on_vscale1_button_release_event),
                    vol_adjustment);
  g_object_set_data_full(G_OBJECT (window1), "vscale1",gtk_widget_ref (vscale1),(GDestroyNotify) gtk_widget_unref);


  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox4, FALSE, TRUE, 8);
  
  closeBtn = gtk_button_new_with_mnemonic(" _close ");
  gtk_widget_show (closeBtn);
  gtk_box_pack_start (GTK_BOX (hbox4), closeBtn, TRUE, FALSE, 0);
  g_signal_connect ((gpointer) closeBtn, "clicked",
                    G_CALLBACK (on_window1_destroy),
                    NULL);


  g_signal_connect ((gpointer) window1, "destroy",
                    G_CALLBACK (on_window1_destroy),
                    NULL);

  g_object_set_data (G_OBJECT (window1), "window1", window1);
  g_object_set_data_full (G_OBJECT (window1), "frame",gtk_widget_ref (frame), (GDestroyNotify) gtk_widget_unref);
  g_object_set_data_full (G_OBJECT (window1), "vbox1",gtk_widget_ref (vbox1), (GDestroyNotify) gtk_widget_unref);
  g_object_set_data_full (G_OBJECT (window1), "hbox1",gtk_widget_ref (hbox1), (GDestroyNotify) gtk_widget_unref);
  g_object_set_data_full (G_OBJECT (window1), "hbox3",gtk_widget_ref (hbox3), (GDestroyNotify) gtk_widget_unref);
  g_object_set_data_full (G_OBJECT (window1), "hbox4",gtk_widget_ref (hbox4), (GDestroyNotify) gtk_widget_unref);
  
  gtk_window_add_accel_group (GTK_WINDOW (window1), accel_group);

  return window1;
}


void get_current_levels() {
  ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_VOLUME), &tmpvol);
  orig.mainvol=tmpvol.left;
  gtk_adjustment_set_value(GTK_ADJUSTMENT(vol_adjustment), (double) tmpvol.left);

}
